@@ -21,10 +21,13 @@ def call(cmd: str) -> None:
 @settings(**g.setting)  # type: ignore
 @given(
     g.hypo_shape(10),
-    g.hypo_vars12(),
+    g.hypo_vars12(skip_info=True),
     st.integers(min_value=1, max_value=3),
 )
 def test_append_ds(shape, v12, rep):
+    for v in v12:
+        for x in v:
+            assert not x[0].endswith("_INFO")
     do_test_append_ds(shape, v12, rep)
 
 
@@ -55,6 +58,10 @@ def do_test_append_ds(shape, v12, rep):
     st.integers(min_value=1, max_value=3),
 )
 def test_to_netcdf_ds(shape, var, rep):
+    do_to_netcdf_ds(shape, var, rep)
+
+
+def do_to_netcdf_ds(shape, var, rep):
     out = "test"
     with tempfile.TemporaryDirectory() as dir:
         dir = dir + "/" + out
@@ -71,4 +78,5 @@ def test_to_netcdf_ds(shape, var, rep):
 
 
 if __name__ == "__main__":
-    do_test_append_ds((2, 2, 2), ([["LG_CELL", 1]], [["LG_CELL", 1]]), 1)
+    do_test_append_ds((2, 2, 2), ([["ENERGY_INFO"]], [["ENERGY_INFO"]]), 1)
+    # do_to_netcdf_ds((1, 1, 1), [["ENERGY_INFO"]], 1)
